@@ -13,31 +13,27 @@
  */
 
 #import <Foundation/Foundation.h>
-#import "GCDAsyncSocket.h"
 #import "RequestManager.h"
+#import "SyncClientProxy.h"
 
-@interface RequestManager : NSObject
+@interface OwnCloudFinderRequestManager : NSObject <SyncClientProxyDelegate>
 {
-	GCDAsyncSocket* _socket;
+	SyncClientProxy *_syncClientProxy;
 
 	NSMutableArray* _requestQueue;
 	NSMutableDictionary* _registeredPathes;
 	NSMutableSet* _requestedPaths;
 
 	NSString *_shareMenuTitle;
-	
-	BOOL _isConnected;
 }
 
 @property (nonatomic, retain) NSString* filterFolder;
 
-+ (RequestManager*)sharedInstance;
++ (OwnCloudFinderRequestManager*)sharedInstance;
 
 - (BOOL)isRegisteredPath:(NSString*)path isDirectory:(BOOL)isDir;
-- (void)askOnSocket:(NSString*)path query:(NSString*)verb;
 - (void)askForIcon:(NSString*)path isDirectory:(BOOL)isDir;
 - (void)menuItemClicked:(NSDictionary*)actionDictionary;
-- (void)start;
 
 - (NSString*) shareItemTitle;
 
