@@ -3,7 +3,8 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; version 2 of the License.
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
@@ -56,6 +57,7 @@ ownCloudGui::ownCloudGui(Application *parent) :
     _settingsDialog(new SettingsDialog(this)),
 #endif
     _logBrowser(0),
+    _contextMenuVisibleOsx(false),
     _recentActionsMenu(0),
     _qdbusmenuWorkaround(false),
     _folderOpenActionMapper(new QSignalMapper(this)),
@@ -66,7 +68,7 @@ ownCloudGui::ownCloudGui(Application *parent) :
     _tray->setParent(this);
 
     // for the beginning, set the offline icon until the account was verified
-    _tray->setIcon( Theme::instance()->folderOfflineIcon(true));
+    _tray->setIcon( Theme::instance()->folderOfflineIcon(/*systray?*/ true, /*currently visible?*/ false));
 
     connect(_tray.data(), SIGNAL(activated(QSystemTrayIcon::ActivationReason)),
             SLOT(slotTrayClicked(QSystemTrayIcon::ActivationReason)));
@@ -92,9 +94,9 @@ ownCloudGui::ownCloudGui(Application *parent) :
              this,SLOT(slotSyncStateChange(Folder*)));
 
     connect( AccountManager::instance(), SIGNAL(accountAdded(AccountState*)),
-             SLOT(setupContextMenu()));
+             SLOT(updateContextMenuNeeded()));
     connect( AccountManager::instance(), SIGNAL(accountRemoved(AccountState*)),
-             SLOT(setupContextMenu()));
+             SLOT(updateContextMenuNeeded()));
 
     connect( Logger::instance(), SIGNAL(guiLog(QString,QString)),
              SLOT(slotShowTrayMessage(QString,QString)));
@@ -158,7 +160,7 @@ void ownCloudGui::slotOpenSettingsDialog()
         }
     } else {
         qDebug() << "No configured folders yet, starting setup wizard";
-        OwncloudSetupWizard::runWizard(qApp, SLOT(slotownCloudWizardDone(int)));
+        slotNewAccountWizard();
     }
 }
 
@@ -172,28 +174,37 @@ void ownCloudGui::slotTrayClicked( QSystemTrayIcon::ActivationReason reason )
         last_click.start();
     }
 
-    // A click on the tray icon should only open the status window on Win and
-    // Linux, not on Mac. They want a menu entry.
-#if !defined Q_OS_MAC
+    // Left click
     if( reason == QSystemTrayIcon::Trigger ) {
-        // Start settings if config is existing.
-        slotOpenSettingsDialog();
-    }
+        if (OwncloudSetupWizard::bringWizardToFrontIfVisible()) {
+            // brought wizard to front
+        } else if (_shareDialogs.size() > 0) {
+            // Share dialog(s) be hidden by other apps, bring them back
+            Q_FOREACH(const QPointer<ShareDialog> &shareDialog, _shareDialogs) {
+                Q_ASSERT(shareDialog.data());
+                raiseDialog(shareDialog);
+            }
+        } else {
+#ifdef Q_OS_MAC
+            // on macOS, a left click always opens menu.
+            // However if the settings dialog is already visible but hidden
+            // by other applications, this will bring it to the front.
+            if (!_settingsDialog.isNull() && _settingsDialog->isVisible()) {
+                raiseDialog(_settingsDialog.data());
+            }
 #else
-    // On Mac, if the settings dialog is already visible but hidden
-    // by other applications, this will bring it to the front.
-    if( reason == QSystemTrayIcon::Trigger ) {
-        if (!_settingsDialog.isNull() && _settingsDialog->isVisible()) {
-            slotShowSettings();
+            slotOpenSettingsDialog();
+#endif
         }
     }
-#endif
+    // FIXME: Also make sure that any auto updater dialogue https://github.com/owncloud/client/issues/5613
+    // or SSL error dialog also comes to front.
 }
 
 void ownCloudGui::slotSyncStateChange( Folder* folder )
 {
     slotComputeOverallSyncStatus();
-    setupContextMenu();
+    updateContextMenuNeeded();
 
     if( !folder ) {
         return; // Valid, just a general GUI redraw was needed.
@@ -215,7 +226,7 @@ void ownCloudGui::slotSyncStateChange( Folder* folder )
 void ownCloudGui::slotFoldersChanged()
 {
     slotComputeOverallSyncStatus();
-    setupContextMenu();
+    updateContextMenuNeeded();
 }
 
 void ownCloudGui::slotOpenPath(const QString &path)
@@ -225,7 +236,7 @@ void ownCloudGui::slotOpenPath(const QString &path)
 
 void ownCloudGui::slotAccountStateChanged()
 {
-    setupContextMenu();
+    updateContextMenuNeeded();
     slotComputeOverallSyncStatus();
 }
 
@@ -261,7 +272,7 @@ void ownCloudGui::slotComputeOverallSyncStatus()
     }
 
     if (!problemAccounts.empty()) {
-        _tray->setIcon(Theme::instance()->folderOfflineIcon(true));
+        _tray->setIcon(Theme::instance()->folderOfflineIcon(true, contextMenuVisible()));
 #ifdef Q_OS_WIN
         // Windows has a 128-char tray tooltip length limit.
         QStringList accountNames;
@@ -288,11 +299,11 @@ void ownCloudGui::slotComputeOverallSyncStatus()
     }
 
     if (allSignedOut) {
-        _tray->setIcon(Theme::instance()->folderOfflineIcon(true));
+        _tray->setIcon(Theme::instance()->folderOfflineIcon(true, contextMenuVisible()));
         _tray->setToolTip(tr("Please sign in"));
         return;
     } else if (allPaused) {
-        _tray->setIcon(Theme::instance()->syncStateIcon(SyncResult::Paused, true));
+        _tray->setIcon(Theme::instance()->syncStateIcon(SyncResult::Paused, true, contextMenuVisible()));
         _tray->setToolTip(tr("Account synchronization is disabled"));
         return;
     }
@@ -322,12 +333,12 @@ void ownCloudGui::slotComputeOverallSyncStatus()
             trayMessage = tr("No sync folders configured.");
         }
 
-        QIcon statusIcon = Theme::instance()->syncStateIcon( overallResult.status(), true);
+        QIcon statusIcon = Theme::instance()->syncStateIcon( overallResult.status(), true, contextMenuVisible());
         _tray->setIcon( statusIcon );
         _tray->setToolTip(trayMessage);
     } else {
         // undefined because there are no folders.
-        QIcon icon = Theme::instance()->syncStateIcon(SyncResult::Problem, true);
+        QIcon icon = Theme::instance()->syncStateIcon(SyncResult::Problem, true, contextMenuVisible());
         _tray->setIcon( icon );
         _tray->setToolTip(tr("There are no sync folders configured."));
     }
@@ -367,7 +378,7 @@ void ownCloudGui::addAccountContextMenu(AccountStatePtr accountState, QMenu *men
             menu->addAction(tr("Managed Folders:"))->setDisabled(true);
         }
 
-        QAction *action = new QAction( tr("Open folder '%1'").arg(folder->shortGuiLocalPath()), this );
+        QAction *action = new QAction( tr("Open folder '%1'").arg(folder->shortGuiLocalPath()), menu );
         connect(action, SIGNAL(triggered()),_folderOpenActionMapper, SLOT(map()));
         _folderOpenActionMapper->setMapping( action, folder->alias() );
         menu->addAction(action);
@@ -399,8 +410,160 @@ void ownCloudGui::addAccountContextMenu(AccountStatePtr accountState, QMenu *men
 
 }
 
+void ownCloudGui::slotContextMenuAboutToShow()
+{
+    // For some reason on OS X _contextMenu->isVisible returns always false
+    qDebug() << "";
+    _contextMenuVisibleOsx = true;
+
+    // Update icon in sys tray, as it might change depending on the context menu state
+    slotComputeOverallSyncStatus();
+}
+
+void ownCloudGui::slotContextMenuAboutToHide()
+{
+    // For some reason on OS X _contextMenu->isVisible returns always false
+    qDebug() << "";
+    _contextMenuVisibleOsx = false;
+
+    // Update icon in sys tray, as it might change depending on the context menu state
+    slotComputeOverallSyncStatus();
+}
+
+bool ownCloudGui::contextMenuVisible() const
+{
+#ifdef Q_OS_MAC
+    return _contextMenuVisibleOsx;
+#else
+    return _contextMenu->isVisible();
+#endif
+}
+
+static bool minimalTrayMenu()
+{
+    static QByteArray var = qgetenv("OWNCLOUD_MINIMAL_TRAY_MENU");
+    return !var.isEmpty();
+}
+
+static bool updateWhileVisible()
+{
+    static QByteArray var = qgetenv("OWNCLOUD_TRAY_UPDATE_WHILE_VISIBLE");
+    if (var == "1") {
+        return true;
+    } else if (var == "0") {
+        return false;
+    } else {
+        // triggers bug on OS X: https://bugreports.qt.io/browse/QTBUG-54845
+        // or flickering on Xubuntu
+        return false;
+    }
+}
+
+static QByteArray forceQDBusTrayWorkaround()
+{
+    static QByteArray var = qgetenv("OWNCLOUD_FORCE_QDBUS_TRAY_WORKAROUND");
+    return var;
+}
+
 void ownCloudGui::setupContextMenu()
 {
+    if (_contextMenu) {
+        return;
+    }
+
+    _contextMenu.reset(new QMenu());
+    _contextMenu->setTitle(Theme::instance()->appNameGUI() );
+
+    _recentActionsMenu = new QMenu(tr("Recent Changes"), _contextMenu.data());
+
+    // this must be called only once after creating the context menu, or
+    // it will trigger a bug in Ubuntu's SNI bridge patch (11.10, 12.04).
+    _tray->setContextMenu(_contextMenu.data());
+
+    // The tray menu is surprisingly problematic. Being able to switch to
+    // a minimal version of it is a useful workaround and testing tool.
+    if (minimalTrayMenu()) {
+        _contextMenu->addAction(_actionQuit);
+        return;
+    }
+
+    // Enables workarounds for bugs introduced in Qt 5.5.0
+    // In particular QTBUG-47863 #3672 (tray menu fails to update and
+    // becomes unresponsive) and QTBUG-48068 #3722 (click signal is
+    // emitted several times)
+    // The Qt version check intentionally uses 5.0.0 (where platformMenu()
+    // was introduced) instead of 5.5.0 to avoid issues where the Qt
+    // version used to build is different from the one used at runtime.
+    // If we build with 5.6.1 or newer, we can skip this because the
+    // bugs should be fixed there.
+#ifdef Q_OS_LINUX
+#if (QT_VERSION >= QT_VERSION_CHECK(5, 0, 0)) && (QT_VERSION < QT_VERSION_CHECK(5, 6, 0))
+    if (qVersion() == QByteArray("5.5.0")) {
+        QObject* platformMenu = reinterpret_cast<QObject*>(_tray->contextMenu()->platformMenu());
+        if (platformMenu
+                && platformMenu->metaObject()->className() == QLatin1String("QDBusPlatformMenu")) {
+            _qdbusmenuWorkaround = true;
+            qDebug() << "Enabled QDBusPlatformMenu workaround";
+        }
+    }
+#endif
+#endif
+
+    if (forceQDBusTrayWorkaround() == "1") {
+        _qdbusmenuWorkaround = true;
+    } else if (forceQDBusTrayWorkaround() == "0") {
+        _qdbusmenuWorkaround = false;
+    }
+
+    // When the qdbusmenuWorkaround is necessary, we can't do on-demand updates
+    // because the workaround is to hide and show the tray icon.
+    if (_qdbusmenuWorkaround) {
+        connect(&_workaroundBatchTrayUpdate, SIGNAL(timeout()), SLOT(updateContextMenu()));
+        _workaroundBatchTrayUpdate.setInterval(30 * 1000);
+        _workaroundBatchTrayUpdate.setSingleShot(true);
+    } else {
+        // Update the context menu whenever we're about to show it
+        // to the user.
+#ifdef Q_OS_MAC
+        // https://bugreports.qt.io/browse/QTBUG-54633
+        connect(_contextMenu.data(), SIGNAL(aboutToShow()), SLOT(slotContextMenuAboutToShow()));
+        connect(_contextMenu.data(), SIGNAL(aboutToHide()), SLOT(slotContextMenuAboutToHide()));
+#else
+        connect(_contextMenu.data(), SIGNAL(aboutToShow()), SLOT(updateContextMenu()));
+#endif
+    }
+
+    // Populate the context menu now.
+    updateContextMenu();
+}
+
+void ownCloudGui::updateContextMenu()
+{
+    if (minimalTrayMenu()) {
+        return;
+    }
+
+    if (_qdbusmenuWorkaround) {
+        // To make tray menu updates work with these bugs (see setupContextMenu)
+        // we need to hide and show the tray icon. We don't want to do that
+        // while it's visible!
+        if (contextMenuVisible()) {
+            if (!_workaroundBatchTrayUpdate.isActive()) {
+                _workaroundBatchTrayUpdate.start();
+            }
+            return;
+        }
+        _tray->hide();
+    }
+
+    _contextMenu->clear();
+    slotRebuildRecentMenus();
+
+    // We must call deleteLater because we might be called from the press in one of the actions.
+    foreach (auto menu, _accountMenus) { menu->deleteLater(); }
+    _accountMenus.clear();
+
+
     auto accountList = AccountManager::instance()->accounts();
 
     bool isConfigured = (!accountList.isEmpty());
@@ -427,41 +590,6 @@ void ownCloudGui::setupContextMenu()
         }
     }
 
-    if ( _contextMenu ) {
-        if (_qdbusmenuWorkaround) {
-            _tray->hide();
-        }
-        _contextMenu->clear();
-        slotRebuildRecentMenus();
-    } else {
-        _contextMenu.reset(new QMenu());
-        _recentActionsMenu = new QMenu(tr("Recent Changes"), _contextMenu.data());
-        // this must be called only once after creating the context menu, or
-        // it will trigger a bug in Ubuntu's SNI bridge patch (11.10, 12.04).
-        _tray->setContextMenu(_contextMenu.data());
-
-        // Enables workarounds for bugs introduced in Qt 5.5.0
-        // In particular QTBUG-47863 #3672 (tray menu fails to update and
-        // becomes unresponsive) and QTBUG-48068 #3722 (click signal is
-        // emitted several times)
-        // The Qt version check intentionally uses 5.0.0 (where platformMenu()
-        // was introduced) instead of 5.5.0 to avoid issues where the Qt
-        // version used to build is different from the one used at runtime.
-#ifdef Q_OS_LINUX
-#if QT_VERSION >= QT_VERSION_CHECK(5, 0, 0)
-        QObject* platformMenu = reinterpret_cast<QObject*>(_tray->contextMenu()->platformMenu());
-        if (platformMenu
-                && platformMenu->metaObject()->className() == QLatin1String("QDBusPlatformMenu")) {
-            _qdbusmenuWorkaround = true;
-            qDebug() << "Enabled QDBusPlatformMenu workaround";
-        }
-#endif
-#endif
-    }
-    _contextMenu->setTitle(Theme::instance()->appNameGUI() );
-    // We must call deleteLater because we might be called from the press in one of the actions.
-    foreach (auto menu, _accountMenus) { menu->deleteLater(); }
-    _accountMenus.clear();
     if (accountList.count() > 1) {
         foreach (AccountStatePtr account, accountList) {
             QMenu* accountMenu = new QMenu(account->account()->displayName(), _contextMenu.data());
@@ -481,6 +609,9 @@ void ownCloudGui::setupContextMenu()
         _contextMenu->addMenu(_recentActionsMenu);
         _contextMenu->addSeparator();
     }
+    if (accountList.isEmpty()) {
+        _contextMenu->addAction(_actionNewAccountWizard);
+    }
     _contextMenu->addAction(_actionSettings);
     if (!Theme::instance()->helpUrl().isEmpty()) {
         _contextMenu->addAction(_actionHelp);
@@ -534,6 +665,31 @@ void ownCloudGui::setupContextMenu()
     }
 }
 
+void ownCloudGui::updateContextMenuNeeded()
+{
+    // For the workaround case updating while visible is impossible. Instead
+    // occasionally update the menu when it's invisible.
+    if (_qdbusmenuWorkaround) {
+        if (!_workaroundBatchTrayUpdate.isActive()) {
+            _workaroundBatchTrayUpdate.start();
+        }
+        return;
+    }
+
+#ifdef Q_OS_MAC
+    // https://bugreports.qt.io/browse/QTBUG-54845
+    // We cannot update on demand or while visible -> update when invisible.
+    if (!contextMenuVisible()) {
+        updateContextMenu();
+    }
+#else
+    if (updateWhileVisible() && contextMenuVisible())
+        updateContextMenu();
+#endif
+
+    // If no update was done here, we might update it on-demand due to
+    // the aboutToShow() signal.
+}
 
 void ownCloudGui::slotShowTrayMessage(const QString &title, const QString &msg)
 {
@@ -580,11 +736,13 @@ void ownCloudGui::setupActions()
     _actionStatus = new QAction(tr("Unknown status"), this);
     _actionStatus->setEnabled( false );
     _actionSettings = new QAction(tr("Settings..."), this);
+    _actionNewAccountWizard = new QAction(tr("New account..."), this);
     _actionRecent = new QAction(tr("Details..."), this);
     _actionRecent->setEnabled( true );
 
     QObject::connect(_actionRecent, SIGNAL(triggered(bool)), SLOT(slotShowSyncProtocol()));
     QObject::connect(_actionSettings, SIGNAL(triggered(bool)), SLOT(slotShowSettings()));
+    QObject::connect(_actionNewAccountWizard, SIGNAL(triggered(bool)), SLOT(slotNewAccountWizard()));
     _actionHelp = new QAction(tr("Help"), this);
     QObject::connect(_actionHelp, SIGNAL(triggered(bool)), SLOT(slotHelp()));
     _actionQuit = new QAction(tr("Quit %1").arg(Theme::instance()->appNameGUI()), this);
@@ -640,18 +798,29 @@ void ownCloudGui::slotUpdateProgress(const QString &folder, const ProgressInfo&
     } else if (progress.totalSize() == 0 ) {
         quint64 currentFile = progress.currentFile();
         quint64 totalFileCount = qMax(progress.totalFiles(), currentFile);
-        _actionStatus->setText( tr("Syncing %1 of %2  (%3 left)")
-            .arg( currentFile ).arg( totalFileCount )
-            .arg( Utility::durationToDescriptiveString2(progress.totalProgress().estimatedEta) ) );
+        QString msg;
+        if (progress.trustEta()) {
+            msg = tr("Syncing %1 of %2  (%3 left)")
+                    .arg( currentFile ).arg( totalFileCount )
+                    .arg( Utility::durationToDescriptiveString2(progress.totalProgress().estimatedEta) );
+        } else {
+            msg = tr("Syncing %1 of %2")
+                    .arg( currentFile ).arg( totalFileCount );
+        }
+        _actionStatus->setText( msg );
     } else {
         QString totalSizeStr = Utility::octetsToString( progress.totalSize() );
-        _actionStatus->setText( tr("Syncing %1 (%2 left)")
-            .arg( totalSizeStr, Utility::durationToDescriptiveString2(progress.totalProgress().estimatedEta) ) );
+        QString msg;
+        if (progress.trustEta()) {
+            msg = tr("Syncing %1 (%2 left)")
+                .arg( totalSizeStr, Utility::durationToDescriptiveString2(progress.totalProgress().estimatedEta) );
+        } else {
+            msg = tr("Syncing %1")
+                .arg( totalSizeStr );
+        }
+        _actionStatus->setText( msg );
     }
 
-
-
-
     _actionRecent->setIcon( QIcon() ); // Fixme: Set a "in-progress"-item eventually.
 
     if (!progress._lastCompletedItem.isEmpty()
@@ -682,7 +851,11 @@ void ownCloudGui::slotUpdateProgress(const QString &folder, const ProgressInfo&
         }
         _recentItemsActions.append(action);
 
-        slotRebuildRecentMenus();
+        // Update the "Recent" menu if the context menu is being shown,
+        // otherwise it'll be updated later, when the context menu is opened.
+        if (updateWhileVisible() && contextMenuVisible()) {
+            slotRebuildRecentMenus();
+        }
     }
 
     if (progress.isUpdatingEstimates()
@@ -732,6 +905,11 @@ void ownCloudGui::slotPauseAllFolders()
     setPauseOnAllFoldersHelper(true);
 }
 
+void ownCloudGui::slotNewAccountWizard()
+{
+    OwncloudSetupWizard::runWizard(qApp, SLOT(slotownCloudWizardDone(int)));
+}
+
 void ownCloudGui::setPauseOnAllFoldersHelper(bool pause)
 {
     QList<AccountState*> accounts;
@@ -745,6 +923,9 @@ void ownCloudGui::setPauseOnAllFoldersHelper(bool pause)
     foreach (Folder* f, FolderMan::instance()->map()) {
         if (accounts.contains(f->accountState())) {
             f->setSyncPaused(pause);
+            if (pause) {
+                f->slotTerminateSync();
+            }
         }
     }
 }
@@ -752,6 +933,7 @@ void ownCloudGui::setPauseOnAllFoldersHelper(bool pause)
 void ownCloudGui::slotShowGuiMessage(const QString &title, const QString &message)
 {
     QMessageBox *msgBox = new QMessageBox;
+    msgBox->setWindowFlags(msgBox->windowFlags() | Qt::WindowStaysOnTopHint);
     msgBox->setAttribute(Qt::WA_DeleteOnClose);
     msgBox->setText(message);
     msgBox->setWindowTitle(title);
