@@ -3,7 +3,8 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; version 2 of the License.
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
@@ -56,10 +57,37 @@ public:
 /**
  * @brief The Account class represents an account on an ownCloud Server
  * @ingroup libsync
+ *
+ * The Account has a name and url. It also has information about credentials,
+ * SSL errors and certificates.
  */
 class OWNCLOUDSYNC_EXPORT Account : public QObject {
     Q_OBJECT
 public:
+    static AccountPtr create();
+    ~Account();
+
+    AccountPtr sharedFromThis();
+
+    /**
+     * The user that can be used in dav url.
+     *
+     * This can very well be different frome the login user that's
+     * stored in credentials()->user().
+     */
+    QString davUser() const;
+    void setDavUser(const QString &newDavUser);
+
+    /// The name of the account as shown in the toolbar
+    QString displayName() const;
+
+    /// The internal id of the account.
+    QString id() const;
+
+    /** Server url of the account */
+    void setUrl(const QUrl &url);
+    QUrl url() const { return _url; }
+
     /**
      * @brief The possibly themed dav path for the account. It has
      *        a trailing slash.
@@ -69,51 +97,19 @@ public:
     void setDavPath(const QString&s) { _davPath = s; }
     void setNonShib(bool nonShib);
 
-    static AccountPtr create();
-    ~Account();
-
-    void setSharedThis(AccountPtr sharedThis);
-    AccountPtr sharedFromThis();
-
-    /// The name of the account as shown in the toolbar
-    QString displayName() const;
-
-    /// The internal id of the account.
-    QString id() const;
-
-    /**
-     * @brief Checks the Account instance is different from @param other
-     *
-     * @returns true, if credentials or url have changed, false otherwise
-     */
-    bool changed(AccountPtr other, bool ignoreUrlProtocol) const;
+    /** Returns webdav entry URL, based on url() */
+    QUrl davUrl() const;
 
     /** Holds the accounts credentials */
     AbstractCredentials* credentials() const;
     void setCredentials(AbstractCredentials *cred);
 
-    /** Server url of the account */
-    void setUrl(const QUrl &url);
-    QUrl url() const { return _url; }
 
-    /** Returns webdav entry URL, based on url() */
-    QUrl davUrl() const;
-
-    /** set and retrieve the migration flag: if an account of a branded
-     *  client was migrated from a former ownCloud Account, this is true
-     */
-    void setMigrated(bool mig);
-    bool wasMigrated();
-
-    QList<QNetworkCookie> lastAuthCookies() const;
-
-    QNetworkReply* headRequest(const QString &relPath);
-    QNetworkReply* headRequest(const QUrl &url);
-    QNetworkReply* getRequest(const QString &relPath);
-    QNetworkReply* getRequest(const QUrl &url);
-    QNetworkReply* deleteRequest( const QUrl &url);
-    QNetworkReply* davRequest(const QByteArray &verb, const QString &relPath, QNetworkRequest req, QIODevice *data = 0);
-    QNetworkReply* davRequest(const QByteArray &verb, const QUrl &url, QNetworkRequest req, QIODevice *data = 0);
+    // For creating various network requests
+    QNetworkReply* sendRequest(const QByteArray &verb,
+                               const QUrl &url,
+                               QNetworkRequest req = QNetworkRequest(),
+                               QIODevice *data = 0);
 
     /** The ssl configuration during the first connection */
     QSslConfiguration getOrCreateSslConfig();
@@ -133,31 +129,40 @@ public:
     // Usually when a user explicitly rejects a certificate we don't
     // ask again. After this call, a dialog will again be shown when
     // the next unknown certificate is encountered.
-    void resetSslCertErrorState();
+    void resetRejectedCertificates();
 
     // pluggable handler
     void setSslErrorHandler(AbstractSslErrorHandler *handler);
 
-    // static helper function
-    static QUrl concatUrlPath(const QUrl &url, const QString &concatPath,
-                              const QList< QPair<QString, QString> > &queryItems = (QList<QPair<QString, QString>>()));
-
-    /**  Returns a new settings pre-set in a specific group.  The Settings will be created
-         with the given parent. If no parent is specified, the caller must destroy the settings */
-    static std::unique_ptr<QSettings> settingsWithGroup(const QString& group, QObject* parent = 0);
-
-    // to be called by credentials only
+    // To be called by credentials only, for storing username and the like
     QVariant credentialSetting(const QString& key) const;
     void setCredentialSetting(const QString& key, const QVariant &value);
 
+    /** Assign a client certificate */
     void setCertificate(const QByteArray certficate = QByteArray(), const QString privateKey = QString());
 
-    void setCapabilities(const QVariantMap &caps);
+    /** Access the server capabilities */
     const Capabilities &capabilities() const;
-    void setServerVersion(const QString &version);
+    void setCapabilities(const QVariantMap &caps);
+
+    /** Access the server version
+     *
+     * For servers >= 10.0.0, this can be the empty string until capabilities
+     * have been received.
+     */
     QString serverVersion() const;
+
+    /** Server version for easy comparison.
+     *
+     * Example: serverVersionInt() >= makeServerVersion(11, 2, 3)
+     *
+     * Will be 0 if the version is not available yet.
+     */
     int serverVersionInt() const;
 
+    static int makeServerVersion(int majorVersion, int minorVersion, int patchVersion);
+    void setServerVersion(const QString &version);
+
     /** Whether the server is too old.
      *
      * Not supporting server versions is a gradual process. There's a hard
@@ -171,20 +176,27 @@ public:
     bool serverVersionUnsupported() const;
 
     // Fixed from 8.1 https://github.com/owncloud/client/issues/3730
+    /** Detects a specific bug in older server versions */
     bool rootEtagChangesNotOnlySubFolderEtags();
 
     void clearCookieJar();
     void lendCookieJarTo(QNetworkAccessManager *guest);
+    QString cookieJarPath();
 
     void resetNetworkAccessManager();
     QNetworkAccessManager* networkAccessManager();
+    QSharedPointer<QNetworkAccessManager> sharedNetworkAccessManager();
 
-    /// Called by network jobs on credential errors.
+    /// Called by network jobs on credential errors, emits invalidCredentials()
     void handleInvalidCredentials();
 
 signals:
+    /// Emitted whenever there's network activity
     void propagatorNetworkActivity();
+
+    /// Triggered by handleInvalidCredentials()
     void invalidCredentials();
+
     void credentialsFetched(AbstractCredentials* credentials);
     void credentialsAsked(AbstractCredentials* credentials);
 
@@ -203,9 +215,11 @@ protected Q_SLOTS:
 
 private:
     Account(QObject *parent = 0);
+    void setSharedThis(AccountPtr sharedThis);
 
     QWeakPointer<Account> _sharedThis;
     QString _id;
+    QString _davUser;
     QMap<QString, QVariant> _settingsMap;
     QUrl _url;
     QList<QSslCertificate> _approvedCerts;
@@ -214,14 +228,15 @@ private:
     QString _serverVersion;
     QScopedPointer<AbstractSslErrorHandler> _sslErrorHandler;
     QuotaInfo *_quotaInfo;
-    QNetworkAccessManager *_am;
-    AbstractCredentials* _credentials;
-    bool _treatSslErrorsAsFailure;
+    QSharedPointer<QNetworkAccessManager> _am;
+    QSharedPointer<AbstractCredentials> _credentials;
+
+    /// Certificates that were explicitly rejected by the user
+    QList<QSslCertificate> _rejectedCertificates;
+
     static QString _configFileName;
-    QByteArray _pemCertificate; 
-    QString _pemPrivateKey;  
+
     QString _davPath; // defaults to value from theme, might be overwritten in brandings
-    bool _wasMigrated;
     friend class AccountManager;
 };
 
