@@ -3,7 +3,8 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; version 2 of the License.
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
@@ -17,19 +18,6 @@
 namespace OCC
 {
 
-void DummyCredentials::syncContextPreInit(CSYNC*)
-{}
-
-void DummyCredentials::syncContextPreStart(CSYNC*)
-{}
-
-bool DummyCredentials::changed(AbstractCredentials* credentials) const
-{
-    DummyCredentials* dummy(dynamic_cast< DummyCredentials* >(credentials));
-
-    return dummy == 0;
-}
-
 QString DummyCredentials::authType() const
 {
     return QString::fromLatin1("dummy");
@@ -56,11 +44,16 @@ bool DummyCredentials::stillValid(QNetworkReply *reply)
     return true;
 }
 
-void DummyCredentials::fetch()
+void DummyCredentials::fetchFromKeychain()
 {
     Q_EMIT(fetched());
 }
 
+void DummyCredentials::askFromUser()
+{
+    Q_EMIT(asked());
+}
+
 void DummyCredentials::persist()
 {}
 
